/* mainRLFM.cpp

   Copyright 2004, V. Makinen.

   An interface to RLFM.cpp.

   This software may be used freely for any non-commercial purpose. However, 
   when distributed, the original source must be clearly stated, and, when the 
   source code is distributed, the copyright notice must be retained and any 
   alterations in the code must be clearly marked. No warranty is given regarding 
   the quality of this software.
*/

#include <stdio.h>
#include <stdlib.h>
#include "386.c"
#include "RLFM.cpp"
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <iostream.h>
#include <sys/stat.h>

char *wbuf; // reading patterns

extern int hz;
extern char *text;

int main(int argc, char *argv[])
{
   int c;
   extern char *optarg;
   extern int optind;

   int nexec = 1;
   int alg = 0;
   int i;

   long tp, te, nf, ti;
   int nw;
   int nbuf, len;
   char *s;   

   while((c = getopt(argc, argv, "a:e:p:r:")) != -1)
      switch(c)
      {
      case 'e':   nexec = atoi(optarg); break;
      case 'p':   print = atoi(optarg); break;
      case 'r':   report = atoi(optarg); break;
      case 'a':   alg = atoi(optarg); break;
      default:   goto usage;
      }
   if((optind >= argc) || (optind+2 < argc)){
usage:
      fprintf(stderr, 
      "\nusage: %s [-a alg] [-e nex] [-p print] [-r report] [string] file\n", argv[0]);
      
      fprintf(stderr,"alg = {0=RLFMindex, 1=FMindex}\n");
      fprintf(stderr,"nex = number of execution times\n");
      fprintf(stderr,"print = {0,1}\n");
      fprintf(stderr,"existence,report,count = {0,1,2}\n\n");
      fprintf(stderr,"Only existence and counting queries are currently supported\n\n");      
      fprintf(stderr,"Examples: \n");
      //fprintf(stderr,"%s -p1 -r1 \"pattern\" RLFM.cpp\n",argv[0]);
      //fprintf(stderr,"  ==> builds RLFMindex over RLFM.cpp and reports occurrences of \"pattern\"\n");
      fprintf(stderr,"%s -a0 -p1 -r2 \"pattern\" RLFM.cpp\n",argv[0]);
      fprintf(stderr,"  ==> builds RLFMindex over RLFM.cpp and counts occurrences of \"pattern\"\n");
      fprintf(stderr,"%s -a1 -p1 -r2 \"pattern\" RLFM.cpp\n",argv[0]);
      fprintf(stderr,"  ==> builds FMindex over RLFM.cpp and counts occurrences of \"pattern\"\n");
      fprintf(stderr,"%s -a0 -e1000 -p0 -r2 RLFM.cpp < patterns\n",argv[0]);
      fprintf(stderr,"  ==> builds RLFMindex over RLFM.cpp and counts occurrences of \n      all lines in file \"patterns\" repeating the search 1000 times\n");
      //fprintf(stderr,"%s -p1 -r10 \"pattern\" RLFM.cpp\n",argv[0]);
      //fprintf(stderr,"  ==> builds RLFMindex over RLFM.cpp and prints the context of \n      each occurrence (10 next symbols)\n");
      fprintf(stderr,"\nTry with larger files to see the real space usage;\non small files, the sublinear structures dominate.\n\n");
      exit(1);
   }
   /* reading the text */
   struct stat sdata;
   stat (argv[argc-1],&sdata);
   n = sdata.st_size;
   if ((c = open(argv[argc-1], 0)) < 0){
      perror(argv[argc-1]);
      exit(1);
   }      
   text = new char[n+1];
   n = 1+read(c, text, n*sizeof(char));
   text[n] = '\0';
   text[n-1] = '\0';
   close(c);    
   if(optind+2 == argc) { // one pattern
      len = strlen(argv[optind]);
      wbuf = (char *)malloc((len+3)*sizeof(char));
      strcpy(wbuf, argv[optind]);
      wbuf[len] = '\0';
      wbuf[len+1] = '\0';
   } 
   else { // many patterns, limited to a few
      wbuf = new char[1000000]; // (char*)malloc(100000*sizeof(char));
      i = 0;
      while((c = getchar()) != EOF && i<1000000-2) {
         wbuf[i++] = (c=='\n')? '\0':c;
      }
      wbuf[i++] = '\0'; wbuf[i] = '\0';
   }
   tp = te = ti = 0;
   TRLFMindex *RLFMindex;
   TFMindex *FMindex;
   nf = nw = 0;
   startclock();        
   if (alg==0)
      RLFMindex = new TRLFMindex(text,n);
   else 
      FMindex = new TFMindex(text,n);
   delete text;
   ti += stopclock();
   startclock();        
   for(s = wbuf; s[0] != '\0';) {
      nw++;
      len = strlen(s);
      m = len;
      for(c = 0; c < nexec; c++)
         if (alg==0)
            nf += RLFMindex->Search(s,m);
	 else 
	    nf += FMindex->Search(s,m);
      s += len;
      /* skip a possible empty word */
      if (s[0] == '\0') s++;
   }   
   te += stopclock();         
   if (alg==0)
      delete RLFMindex;
   else
      delete FMindex;
   delete wbuf;
   printf("w=%d match=%ld \n i=%4fs e=%.4fs(%ldms/%d)\n",
         nw, (long)nf/nexec,
        (ti*1.0)/(hz*1.0),
      (te*1.0)/(hz*nexec),
      te, nexec);
   return 0;
}
